/**
* @file utils.cpp
*
* Copyright (C) 2020. Huawei Technologies Co., Ltd. All rights reserved.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*/
#include "utils.h"
#include <iostream>
#include <fstream>
#include <cstring>
#include <vector>
#include "acl/acl.h"
#include "/usr/local/mdc_sdk_llvm/dp_gea/mdc_cross_compiler/sysroot/usr/include/opencv4/opencv2/imgcodecs.hpp"
#include "/usr/local/mdc_sdk_llvm/dp_gea/mdc_cross_compiler/sysroot/usr/include/opencv4/opencv2/imgproc.hpp"
#if defined(_MSC_VER)
#include <windows.h>
#else

#include <sys/stat.h>

#endif


extern bool g_isDevice;
aclrtRunMode Utils::runMode_ = ACL_DEVICE;

bool Utils::validStatus(const char *msg, const aclError &status) {
    if (status == ACL_SUCCESS) {
        INFO_LOG("[%s]", msg);
        return true;
    } else {
        ERROR_LOG("[%s], code: [%d], description:\n%s",
                  msg, static_cast<int32_t>(status), aclGetRecentErrMsg());
        return false;
    }
    return false;
}


Result
Utils::MemcpyVariableToDeviceBuffer(std::vector<std::vector<std::vector<float>>> VectorVariable, void *&picDevBuffer,
                                    size_t inputBuffSize) {
    int u = 0;
    void *inputBuff = nullptr;
    // Flatten the 3D vector into a 1D vector
    std::vector<float> VectorVariable_one_dimension;
    for (const auto &vec2D: VectorVariable) {
        for (const auto &vec1D: vec2D) {
            VectorVariable_one_dimension.insert(VectorVariable_one_dimension.end(), vec1D.begin(),
                                                vec1D.end());
        }
    }


    // uint32_t fileSize = 0;
    // auto ret = Utils::ReadBinFile(fileName, inputBuff, fileSize);
    inputBuff = static_cast<void *>(VectorVariable_one_dimension.data());

    aclError aclRet = aclrtMemcpy(picDevBuffer, inputBuffSize, inputBuff, inputBuffSize, ACL_MEMCPY_DEVICE_TO_DEVICE);
    if (aclRet != ACL_SUCCESS) {
        ERROR_LOG("memcpy d2d failed. buffer size is %zu, errorCode is %d", inputBuffSize,
                  static_cast<int32_t>(aclRet));
        (void) aclrtFree(inputBuff);
        return FAILED;
    }
    (void) aclrtFree(inputBuff);

    return SUCCESS;
}


Result Utils::CheckPathIsFile(const std::string &fileName) {
#if defined(_MSC_VER)
    DWORD bRet = GetFileAttributes((LPCSTR)fileName.c_str());
    if (bRet == FILE_ATTRIBUTE_DIRECTORY) {
        ERROR_LOG("%s is not a file, please enter a file", fileName.c_str());
        return FAILED;
    }
#else
    struct stat sBuf;
    int fileStatus = stat(fileName.data(), &sBuf);
    if (fileStatus == -1) {
        ERROR_LOG("failed to get file");
        return FAILED;
    }
    if (S_ISREG(sBuf.st_mode) == 0) {
        ERROR_LOG("%s is not a file, please enter a file", fileName.c_str());
        return FAILED;
    }
#endif
    return SUCCESS;
}
void* Utils::CopyDataToDevice(void* data, uint32_t dataSize, aclrtMemcpyKind policy) {
    void* buffer = nullptr;
    aclError aclRet = aclrtMalloc(&buffer, dataSize, ACL_MEM_MALLOC_HUGE_FIRST);
    if (aclRet != ACL_SUCCESS) {
        ERROR_LOG("malloc device data buffer failed, aclRet is %d", aclRet);
        return nullptr;
    }

    aclRet = aclrtMemcpy(buffer, dataSize, data, dataSize, policy);
    if (aclRet != ACL_SUCCESS) {
        ERROR_LOG("Copy data to device failed, aclRet is %d", aclRet);
        (void)aclrtFree(buffer);
        return nullptr;
    }

    return buffer;
}

bool Utils::PreProcess(std::shared_ptr<ImageDesc>& imageData, const std::string& imageFile) {
    //Read image using opencv
//    cv::Mat image = cv::imread(imageFile, cv::CV_LOAD_IMAGE_COLOR);
    cv::Mat image = cv::imread(imageFile);
    if (image.empty()) {
        ERROR_LOG("Read image %s failed", imageFile.c_str());
        return false;
    }
    //resize image to model size
//    cv::Mat reiszedImage;
//    cv::resize(image, reiszedImage, cv::Size(MODEL_INPUT_WIDTH,MODEL_INPUT_HEIGHT));

    uint32_t size = IMAGE_CHAN_SIZE_U8(1600, 900);

    aclError ret = aclrtGetRunMode(&runMode_);
    if (ret != ACL_SUCCESS) {
        ERROR_LOG("acl get run mode failed");
    }

    void* imageDev;
    //copy image data to device
    if (runMode_ == ACL_HOST) {
        imageDev = Utils::CopyDataHostToDevice(image.ptr<u_int8_t>(0), size);
        if (imageDev == nullptr) {
            ERROR_LOG("Copy image info to device failed");
            return FAILED;
        }
    }
    else {
        imageDev = Utils::CopyDataDeviceToDevice(image.ptr<u_int8_t>(0), size);
        if (imageDev == nullptr) {
            ERROR_LOG("Copy image info to device failed");
            return FAILED;
        }
    }

    imageData->size = size;
    imageData->data.reset((uint8_t *)imageDev,
                          [](uint8_t* p) { aclrtFree(p);} );

    return true;
}
void* Utils::CopyDataDeviceToDevice(void* deviceData, uint32_t dataSize) {
    return CopyDataToDevice(deviceData, dataSize, ACL_MEMCPY_DEVICE_TO_DEVICE);
}

void* Utils::CopyDataHostToDevice(void* deviceData, uint32_t dataSize) {
    return CopyDataToDevice(deviceData, dataSize, ACL_MEMCPY_HOST_TO_DEVICE);
}
